import React from 'react'
import qs from 'querystring'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { Icon, th } from '@pubsweet/ui'
import { compose, withHandlers } from 'recompose'

const createAnchorElement = (file, filename) => {
  const url = URL.createObjectURL(file)
  const a = document.createElement('a')

  a.href = url
  a.download = filename
  document.body.appendChild(a)

  return {
    a,
    url,
  }
}

const removeAnchorElement = (a, url) => {
  document.body.removeChild(a)
  URL.revokeObjectURL(url)
}

const FileDownload = ({ downloadFile }) => (
  <IconButton onClick={downloadFile}>
    <Icon primary size={3}>
      download
    </Icon>
  </IconButton>
)

export default compose(
  connect(state => ({
    token: state.currentUser.user.token,
  })),
  withHandlers({
    downloadFile: ({ fileId, token, fileName = 'file' }) => () => {
      const fileURL = `${
        window.location.origin
      }/api/files/${fileId}?${qs.stringify({
        download: true,
      })}`

      const xhr = new XMLHttpRequest()
      xhr.onreadystatechange = function onXhrStateChange() {
        if (this.readyState === 4) {
          if (this.status >= 200 && this.status < 300) {
            const f = new File([this.response], fileName)

            const { a, url } = createAnchorElement(f, fileName)
            a.click()
            removeAnchorElement(a, url)
          }
        }
      }
      xhr.open('GET', fileURL)
      xhr.responseType = 'blob'
      xhr.setRequestHeader('Authorization', `Bearer ${token}`)
      xhr.send()
    },
  }),
)(FileDownload)

// #region styled-components
const IconButton = styled.div`
  align-items: center;
  cursor: pointer;
  display: flex;
  justify-content: center;
  margin: 0 ${th('subGridUnit')};
  &:hover {
    opacity: 0.7;
  }
`
// #endregion
